function [E, theta, b] = minPMAC(G, bu, w);
% This is the ellipsoid method part and the main function to call.

% the inputs are: 
% 1)  G, an Ly by U by N channel matrix. Ly is the number of receiver antennas, 
%     U is the total number of users and N is the total number of tones.
%     G(:,:,n) is the channel matrix for all users on tone n
%     and G(:,u,n) is the channel for user u on tone n. In this code we assume each user 
%     only has single transmit antenna, thus G(:,u,n) is a column vector. 
% 2)  w, a U by 1 vector containing the weights for each user's power.
% 3)  bu, a U by 1 vector containing the target rates for all the users.

% the outputs are:
% 1)  E, a U by N matrix containing the powers for all users and over all tones
%     that minimizes the weighted-sum power. E(u,:) is the power allocation for
%     user u over all tones.
% 2)  theta, the optimal U by 1 dual variable vector containing optimal weights
%     of rates. theta determines the decoding order.
% 3)  b, a U by N matrix containing the rates of all users on all tones after convergence.


bu = bu  * log(2);                       % conversion from bits to nuts 
err = 1e-6;                              % error tolerance
w_0 = 1000;                              % Very large starting ellipsoid, in Problem 13.11 
count = 0;                               % you are asked to implement the initialization for staring ellipsoid
[Ly, U, N] = size(G);

theta = w_0 * ones(U,1);
A = eye(U) * (U * w_0^2);                % starting ellipsoid
g = w_0 * ones(U,1);


while sqrt(g' * A * g) > err             % stopping criteria
    
    ind = find(theta < zeros(U,1));
    
    while ~isempty(ind)                  % This part is to make sure that theta is feasible,
        g = zeros(U,1);                  % it was not covered in the lecture notes and you may skip this part
        g(ind(1)) = -1;
        gt = g / sqrt(g' * A * g);
        theta = theta - 1 / (U + 1) * A * gt;
        A = U^2 / (U^2 - 1) * (A - 2 / (U + 1) * A * gt * gt' * A);
        ind = find(theta < zeros(U,1));
    end                                  % Ellipsoid method starts here
    [f, b, E] = Lag_dual_f(G, theta, w, bu);
    g = sum(b,2) - bu;                   % sub-gradient
    gt = g / sqrt(g' * A * g);
    theta = theta - 1 / (U + 1) * A * gt;
    A = U^2 / (U^2 - 1) * (A - 2 / (U + 1) * A * gt * gt' * A);
    count = count+1
end

b = b /log(2);                            % conversion from nuts to bits
   
    
    
        

    